structure Verilog_ast =
struct
    exception WrongAST
    (*
	+,* in regular expression -> list
	? in regular expression -> option

	param -> parameter
	decl -> declaration
	expr -> expression
	inst -> instantiation
	cont -> continuous
	stmt -> statement
	proc -> procedural
	ctrl -> control
	cond -> conditional
	term -> terminal

	SSupply0 -> supply0 of strength0
	SSupply1 -> supply1 of strength1
	NSupply0 -> supply0 of net_type
	NSupply1 -> supply1 of net_type

	U->unary, B->binary, L->logical,
	if not logical, bitwise or reduction
    *)
    type program = source_text
     and always_statement = statement
     and assignment = lvalue * expression
     and binary_operator = BAdd | BSub | BMul | BDiv | BMod
			 | BEq | BNEq | BCaseEq | BCaseNEq
			 | BLAnd | BLOr
			 | BLess | BLessEq | BGreater | BGreaterEq
			 | BAnd | BOr | BXor | BXnor | BRShift | BLShift
     and block_item_declaration = declaration
     and blocking_assignment = lvalue * delay_or_event_control option * expression
     and case_item = expression list option * statement_or_null
     and case_statement = case_type * expression * case_item list
     and case_type = Case | CaseZ | CaseX
     and charge_strength = Small | Medium | Large
     and combinational_entry = level_input_list * output_symbol
     and concatenation = expression list
     and conditional_statement = expression * statement_or_null * statement_or_null option
     and constant_expression = expression
     and continuous_assign = drive_strength option * delay option * list_of_assignments
     and declaration = ParamDecl of parameter_declaration
		     | InputDecl of input_declaration
		     | OutputDecl of output_declaration
		     | InoutDecl of inout_declaration
		     | NetDecl of net_declaration
		     | RegDecl of reg_declaration
		     | IntegerDecl of integer_declaration
		     | RealDecl of real_declaration
		     | TimeDecl of time_declaration
		     | RealtimeDecl of realtime_declaration
		     | EventDecl of event_declaration
     and delay = NumDelay of number
	       | IDDelay of identifier
	       | MintypmaxDelay of mintypmax_expression * mintypmax_expression option * mintypmax_expression option
     and delay_control = NumDelayCtrl of number
		       | IDDelayCtrl of identifier
		       | MintypmaxDelayCtrl of mintypmax_expression
     and delay_or_event_control = DelayCtrl of delay_control
				| EventCtrl of event_control
				| RepeatEventCtrl of expression * event_control
     and description = ModuleDecl of module_declaration
		     | UdpDecl of udp_declaration
     and disable_statement = name_of_block_or_task
     and drive_strength = strength0 * strength1 * bool (* true->0,1 false->1,0 *)
     and edge = Edge1 of level_symbol * level_symbol
	      | Edge2 of edge_symbol
     and edge_input_list = level_symbol list * edge * level_symbol list
     and edge_symbol = symbol
     and event_control = IDEventCtrl of identifier
		       | ExprEventCtrl of event_expression
     and event_declaration = name_of_event list
     and event_expression = ExprEventExpr of expression
			  | PosedgeEventExpr of scalar_event_expression
			  | NegedgeEventExpr of scalar_event_expression
			  | OrEventExpr of event_expression * event_expression
     and event_trigger = name_of_event
     and expandrange = bool option * range (* None->range, Some(true)->vectored, Some(false)->scalared *)
     and expression = PrimaryExpr of primary
		    | UnaryExpr of unary_operator * primary
		    | BinaryExpr of binary_operator * expression * expression
		    | TernaryExpr of expression * expression * expression (* _ ? _ : _ *)
		    | StringExpr of string
     and function = range_or_type option * name_of_function * function_item_declaration list * statement
     and function_call = FunCall of name_of_function * expression list
		       | SystemFunCall of name_of_system_function * expression list
     and function_item_declaration = declaration
     and gate_instance = name_of_gate_instance option * terminal list
     and gate_instantiation = gate_type * drive_strength option * delay option * gate_instance list
     and gate_type = AndGate | NandGate | OrGate | NorGate | XorGate | XnorGate
		   | BufGate | NotGate
		   | Bufif0Gate | Bufif1Gate | Notif0Gate | Notif1Gate
		   | NmosGate | PmosGate | RnmosGate | RpmosGate
		   | TranGate | RtranGate
		   | Tranif0Gate | Tranif1Gate | Rtranif0Gate | Rtranif1Gate
		   | CmosGate | RcmosGate
		   | PullupGate | PulldownGate
     and identifier = single_identifier list
     and init_val = number
     and initial_statement = statement
     and inout_declaration = range option * list_of_variables
     and input_declaration = range option * list_of_variables
     and input_list = LevelInput of level_input_list
		    | EdgeInput of edge_input_list
     and integer_declaration = list_of_register_variables
     and level_input_list = level_symbol list
     and level_symbol = symbol
     and list_of_assignments = assignment list
     and list_of_module_connections = NamedPortList of named_port_connection list
				    | OrderedPortList of ordered_port_connection list
     and list_of_param_assignments = param_assignment list
     and list_of_ports = port list
     and list_of_register_variables = register_variable list
     and list_of_variables = name_of_variable list
     and loop_statement = Forever of statement
			| Repeat of expression * statement
			| While of expression * statement
			| For of assignment * expression * assignment * statement
     and lvalue = IDLvalue of identifier
		| IDIndexLvalue of identifier * expression
		| IDRangeLvalue of identifier * constant_expression * constant_expression
		| ConcatLvalue of concatenation
     and mintypmax_expression = expression * ( expression * expression ) option
     and module_declaration = module_keyword * name_of_module * list_of_ports option * module_item list
     and module_instance = name_of_instance * list_of_module_connections
     and module_instantiation = name_of_module * parameter_value_assignment option * module_instance list
     and module_item = Declaration of declaration
		     | GateInst of gate_instantiation
		     | UdpInst of udp_instantiation
		     | ModuleInst of module_instantiation
		     | ParamOverride of parameter_override
		     | ContAssign of continuous_assign
		     | SpecifyBlock of specify_block
		     | InitialStmt of initial_statement
		     | AlwaysStmt of always_statement
		     | Task of task
		     | Function of function
     and module_keyword = Module | MacroModule
     and multiple_concatenation = expression list
     and name_of_block = single_identifier
     and name_of_block_or_task = single_identifier
     and name_of_event = single_identifier
     and name_of_function = single_identifier
     and name_of_gate_instance = single_identifier * range option
     and name_of_instance = single_identifier * range option
     and name_of_memory = single_identifier
     and name_of_module = single_identifier
     and name_of_port = single_identifier
     and name_of_register = single_identifier
     and name_of_system_function = system_identifier
     and name_of_system_task = system_identifier
     and name_of_task = single_identifier
     and name_of_udp = single_identifier
     and name_of_variable = single_identifier
     and named_port_connection = single_identifier * expression option
     and net_declaration = NetDecl1 of net_type * expandrange option * delay option * list_of_variables
			 | NetDecl2 of charge_strength option * expandrange option * delay option * list_of_variables
			 | NetDecl3 of net_type * drive_strength option * expandrange option * delay option * list_of_assignments
     and net_type = Wire | Tri | Tri1 | NSupply0 | Wand
		  | Triand | Tri0 | NSupply1 | Wor | Trior
     and next_state = symbol
     and non_blocking_assignment = lvalue * delay_or_event_control option * expression
     and number = string
     and ordered_port_connection = expression option
     and output_declaration = range option * list_of_variables
     and output_symbol = symbol
     and output_terminal_name = name_of_variable
     and param_assignment = identifier * constant_expression
     and parameter_declaration = list_of_param_assignments
     and parameter_override = list_of_param_assignments
     and parameter_value_assignment = expression list
     and par_block = ( name_of_block * block_item_declaration list ) option * statement list
     and port = name_of_port option * port_expression option
     and port_expression = port_reference list
     and port_reference = name_of_variable * constant_expression option * constant_expression option
     and primary = NumPrimary of number
		 | IDPrimary of identifier
		 | IDIndexPrimary of identifier * expression
		 | IDRangePrimary of identifier * constant_expression * constant_expression
		 | ConcatPrimary of concatenation
		 | MulConcatPrimary of multiple_concatenation
		 | FunCallPrimary of function_call
		 | MintypmaxPrimary of mintypmax_expression
     and procedural_continuous_assignment = Assign of assignment
					  | Deassign of lvalue
					  | Force of assignment
					  | Release of lvalue
     and procedural_timing_control_statement = delay_or_event_control * statement_or_null
     and range = constant_expression * constant_expression
     and range_or_type = Range of range | Integer | Real | Realtime | Time
     and real_declaration = list_of_variables
     and realtime_declaration = list_of_variables
     and reg_declaration = range option * list_of_register_variables
     and register_variable = Register of name_of_register * expression option (* ICARUS Verilog extension? *)
			   | Memory of name_of_memory * constant_expression * constant_expression
     and scalar_event_expression = expression
     and seq_block = ( name_of_block * block_item_declaration list ) option * statement list
     and sequential_entry = input_list * state * next_state
     and single_identifier = string
     and source_text = description list
     and specify_block = specify_item list
     and specify_item = unit
     and state = level_symbol
     and statement = BlockAssign of blocking_assignment
		   | NonBlockAssign of non_blocking_assignment
		   | ProcContAssign of procedural_continuous_assignment
		   | ProcTimingCtrlStmt of procedural_timing_control_statement
		   | CondStmt of conditional_statement
		   | CaseStmt of case_statement
		   | LoopStmt of loop_statement
		   | WaitStmt of wait_statement
		   | DisableStmt of disable_statement
		   | EventTrigger of event_trigger
		   | SeqBlock of seq_block
		   | ParBlock of par_block
		   | TaskEnable of task_enable
		   | SystemTaskEnable of system_task_enable
     and statement_or_null = statement option
     and strength0 = SSupply0 | Strong0 | Pull0 | Weak0 | Highz0
     and strength1 = SSupply1 | Strong1 | Pull1 | Weak1 | Highz1
     and symbol = Symbol0 | Symbol1 | SymbolX
		| SymbolQ | SymbolB | SymbolM (* minus *)
		| SymbolR | SymbolF | SymbolP | SymbolN | SymbolA (* asterisk *)
     and system_identifier = string
     and system_task_enable = name_of_system_task * expression list
     and table_definition = table_entries
     and table_entries = CombEntry of combinational_entry list
		       | SeqEntry of sequential_entry list
     and task = name_of_task * task_item_declaration list * statement_or_null
     and task_enable = name_of_task * expression list
     and task_item_declaration = declaration
     and terminal = expression
     and time_declaration = list_of_register_variables
     and udp_declaration = name_of_udp * name_of_variable list * udp_port_declaration list * udp_initial_statement option * table_definition
     and udp_initial_statement = output_terminal_name * init_val
     and udp_instantiation = unit
     and udp_port_declaration = declaration
     and unary_operator = UPlus | UMinus | ULNot | UNot | UAnd
			| UNand | UOr | UNor | UXor | UXnor
     and wait_statement = expression * statement_or_null

    fun print pgm =
	let
	    (* s->string, n->newline, i->indent, o->opt, l->list *)
	    fun ps s = print_string s
	    fun pn() = print_newline()
	    fun psn s = ps s ; pn()
	    fun pi 0 = ()
	      | pi depth = ps "  " ; pi(depth-1)
	    fun po pfn None = ()
	      | po pfn Some(opt) = ( ps " " ; pfn opt )
	    fun pl pfn sep plist =
		let
		    fun t [] = ()
		      | t [h] = pfn h
		      | t (h::l) = pfn h ; ps sep ; t l
		in
		    if plist=[] then ()
		    else t plist
		end

	    fun program arg = source_text arg
	    and always_statement d stmt = pi d ; psn "always " ; statement (d+1) stmt
	    and assignment (lv,exp) = lvalue lv ; ps "=" ; expression exp
	    and binary_operator BAdd = ps "+"
	      | binary_operator BSub = ps "-"
	      | binary_operator BMul = ps "*"
	      | binary_operator BDiv = ps "/"
	      | binary_operator BMod = ps "%"
	      | binary_operator BEq = ps "=="
	      | binary_operator BNEq = ps "!="
	      | binary_operator BCaseEq = ps "==="
	      | binary_operator BCaseNEq = ps "!=="
	      | binary_operator BLAnd = ps "&&"
	      | binary_operator BLOr = ps "||"
	      | binary_operator BLess = ps "<"
	      | binary_operator BLessEq = ps "<="
	      | binary_operator BGreater = ps ">"
	      | binary_operator BGreaterEq = ps ">="
	      | binary_operator BAnd = ps "&"
	      | binary_operator BOr = ps "|"
	      | binary_operator BXor = ps "^"
	      | binary_operator BXnor = ps "~^"
	      | binary_operator BRShift = ps ">>"
	      | binary_operator BLShift = ps "<<"
	    and block_item_declaration d ParamDecl(i) = parameter_declaration d i
	      | block_item_declaration d RegDecl(i) = reg_declaration d i
	      | block_item_declaration d IntegerDecl(i) = integer_declaration d i
	      | block_item_declaration d RealDecl(i) = real_declaration d i
	      | block_item_declaration d TimeDecl(i) = time_declaration d i
	      | block_item_declaration d RealtimeDecl(i) = realtime_declaration d i
	      | block_item_declaration d EventDecl(i) = event_declaration d i
	      | block_item_declaration _ _ = raise WrongAST
	    and blocking_assignment d (lv,ctl,expr) = pi d ; lvalue lv ; ps "=" ; po delay_or_event_control ctl ; ps " " ; expression expr
	    and case_item d (Some(e),s) = pi d ; pl expression ", " e ; psn " : " ; statement_or_null d s
	      | case_item d (None,s) = pi d ; psn " : " ; statement_or_null d s
	    and case_statement d (t,e,l) = pi d ; case_type t ; ps "(" ; expression e ; psn ")" ; pl (case_item (d+1)) "\n" l ; psn "endcase"
	    and case_type Case = ps "case"
	      | case_type CaseZ = ps "casez"
	      | case_type CaseX = ps "casex"
	    and charge_strength Small = ps "(small)"
	      | charge_strength Medium = ps "(medium)"
	      | charge_strength Large = ps "(large)"
	    and combinational_entry _ _ = ()
	    and concatenation _ = ()
	    and conditional_statement d (e,s,None) = pi d ; ps "if (" ; expression e ; psn ")" ; statement_or_null (d+1) s
	      | conditional_statement d (e,s1,Some(s2)) = pi d ; ps "if (" ; expression e ; psn ")" ; statement_or_null (d+1) s1 ; pi d ; psn "else" ; statement_or_null (d+1) s2
	    and constant_expression e = expression e
	    and continuous_assign d (s,l,a) = pi d ; ps "assign " ; po drive_strength s ; po delay l ; list_of_assignments a ; psn ";"
	    and declaration d ParamDecl(a) = parameter_declaration d a
	      | declaration d InputDecl(a) = input_declaration d a
	      | declaration d OutputDecl(a) = output_declaration d a
	      | declaration d InoutDecl(a) = inout_declaration d a
	      | declaration d NetDecl(a) = net_declaration d a
	      | declaration d RegDecl(a) = reg_declaration d a
	      | declaration d IntegerDecl(a) = integer_declaration d a
	      | declaration d RealDecl(a) = real_declaration d a
	      | declaration d TimeDecl(a) = time_declaration d a
	      | declaration d RealtimeDecl(a) = realtime_declaration d a
	      | declaration d EventDecl(a) = event_declaration d a
	    and delay _ = ()
	    and delay_control NumDelayCtrl(n) = ps "#" ; number n
	      | delay_control IDDelayCtrl(i) = ps "#" ; identifier i
	      | delay_control MintypmaxDelayCtrl(m) = ps "#(" ; mintypmax_expression m ; ps ")"
	    and delay_or_event_control DelayCtrl(c) = delay_control c
	      | delay_or_event_control EventCtrl(c) = event_control c
	      | delay_or_event_control RepeatEventCtrl(e,c) = ps "repeat(" ; expression e ; ps ")" ; event_control c
	    and description d ModuleDecl(m) = module_declaration d m
	      | description d UdpDecl(u) = udp_declaration d u
	    and disable_statement _ _ = ()
	    and drive_strength (s0,s1,true) = ps "(" ; strength0 s0 ; ps "," ; strength1 s1 ; ps ")"
	      | drive_strength (s0,s1,false) = ps "(" ; strength1 s1 ; ps "," ; strength0 s0 ; ps ")"
	    and edge _ _ = ()
	    and edge_input_list _ _ = ()
	    and edge_symbol _ _ = ()
	    and event_control IDEventCtrl(i) = ps "@" ; identifier i
	      | event_control ExprEventCtrl(e) = ps "@(" ; event_expression e ; ps ")"
	    and event_declaration _ _ = ()
	    and event_expression ExprEventExpr(e) = expression e
	      | event_expression PosedgeEventExpr(e) = ps "posedge " ; scalar_event_expression e
	      | event_expression NegedgeEventExpr(e) = ps "negedge " ; scalar_event_expression e
	      | event_expression OrEventExpr(e1,e2) = event_expression e1 ; ps " or " ; event_expression e2
	    and event_trigger _ _ = ()
	    and expandrange _ _ = ()
	    and expression PrimaryExpr(p) = primary p
	      | expression UnaryExpr(u,p) = unary_operator u ; primary p
	      | expression BinaryExpr(b,e1,e2) = expression e1 ; binary_operator b; expression e2
	      | expression TernaryExpr(e1,e2,e3) = expression e1 ; ps "?" ; expression e2 ; ps ":" ; expression e3
	      | expression StringExpr(s) = ps s
	    and function _ _ = ()
	    and function_call _ = ()
	    and function_item_declaration _ _ = ()
	    and gate_instance _ _ = ()
	    and gate_instantiation _ _ = ()
	    and gate_type _ _ = ()
	    and identifier l = pl single_identifier "." l
	    and init_val _ _ = ()
	    and initial_statement d s = pi d ; psn "initial" ; statement (d+1) s
	    and inout_declaration d (r,l) = pi d ; ps "inout" ; po range r ; ps " " ; list_of_variables l ; psn ";"
	    and input_declaration d (r,l) = pi d ; ps "input" ; po range r ; ps " " ; list_of_variables l ; psn ";"
	    and input_list _ _ = ()
	    and integer_declaration _ _ = ()
	    and level_input_list _ _ = ()
	    and level_symbol _ _ = ()
	    and list_of_assignments l = pl assignment ", " l
	    and list_of_module_connections _ _ = ()
	    and list_of_param_assignments _ _ = ()
	    and list_of_ports _ _ = ()
	    and list_of_register_variables r = pl register_variable ", " r
	    and list_of_variables l = pl name_of_variable ", " l
	    and loop_statement _ _ = ()
	    and lvalue IDLvalue(i) = identifier i
	      | lvalue IDIndexLvalue(i,e) = identifier i ; ps "[" ; expression e ; ps "]"
	      | lvalue IDRangeLvalue(i,e1,e2) = identifier i ; ps "[" ; constant_expression e1 ; ps ": " ; constant_expression e2 ; ps "]"
	      | lvalue ConcatLvalue(c) = concatenation c
	    and mintypmax_expression _ = ()
	    and module_declaration d (k,n,p,i) = pi d ; module_keyword d k ; ps " " ; name_of_module n ; po (list_of_ports d) p ; psn ";" ; pl (module_item (d+1)) "" i ; pi d ; psn "endmodule"
	    and module_instance _ _ = ()
	    and module_instantiation _ _ = ()
	    and module_item d Declaration(decl) = declaration d decl
	      | module_item d GateInst(g) = gate_instantiation d g
	      | module_item d UdpInst(u) = udp_instantiation d u
	      | module_item d ModuleInst(m) = module_instantiation d m
	      | module_item d ParamOverride(p) = parameter_override d p
	      | module_item d ContAssign(a) = continuous_assign d a
	      | module_item d SpecifyBlock(s) = specify_block d s
	      | module_item d InitialStmt(s) = initial_statement d s
	      | module_item d AlwaysStmt(s) = always_statement d s
	      | module_item d Task(t) = task d t
	      | module_item d Function(f) = function d f
	    and module_keyword d Module = ps "module"
	      | module_keyword d MacroModule = ps "macromodule"
	    and multiple_concatenation _ = ()
	    and name_of_block n = single_identifier n
	    and name_of_block_or_task n = single_identifier n
	    and name_of_event n = single_identifier n
	    and name_of_function n = single_identifier n
	    and name_of_gate_instance _ _ = ()
	    and name_of_instance _ _ = ()
	    and name_of_memory n = single_identifier n
	    and name_of_module n = single_identifier n
	    and name_of_port n = single_identifier n
	    and name_of_register n = single_identifier n
	    and name_of_system_function n = system_identifier n
	    and name_of_system_task n = system_identifier n
	    and name_of_task n = single_identifier n
	    and name_of_udp n = single_identifier n
	    and name_of_variable n = single_identifier n
	    and named_port_connection _ _ = ()
	    and net_declaration _ _ = ()
	    and net_type _ _ = ()
	    and next_state _ _ = ()
	    and non_blocking_assignment d (lv,ctl,expr) = pi d ; lvalue lv ; ps "<=" ; po delay_or_event_control ctl ; ps " " ; expression expr
	    and number n = ps n
	    and ordered_port_connection _ _ = ()
	    and output_declaration d (r,l) = pi d ; ps "output" ; po range r ; ps " " ; list_of_variables l ; psn ";"
	    and output_symbol _ _ = ()
	    and output_terminal_name _ _ = ()
	    and param_assignment _ _ = ()
	    and parameter_declaration _ _ = ()
	    and parameter_override _ _ = ()
	    and parameter_value_assignment _ _ = ()
	    and par_block _ _ = ()
	    and port _ _ = ()
	    and port_expression _ _ = ()
	    and port_reference _ _ = ()
	    and primary NumPrimary(n) = number n
	      | primary IDPrimary(i) = identifier i
	      | primary IDIndexPrimary(i,e) = identifier i ; ps "[" ; expression e ; ps "]"
	      | primary IDRangePrimary(i,e1,e2) = identifier i ; ps "[" ; constant_expression e1 ; ps ":" ; constant_expression e2 ; ps "]"
	      | primary ConcatPrimary(c) = concatenation c
	      | primary MulConcatPrimary(m) = multiple_concatenation m
	      | primary FunCallPrimary(f) = function_call f
	      | primary MintypmaxPrimary(e) = mintypmax_expression e
	    and procedural_continuous_assignment d Assign(p) = pi d ; ps "assign " ; assignment p
	      | procedural_continuous_assignment d Deassign(p) = pi d ; ps "deassign " ; lvalue p
	      | procedural_continuous_assignment d Force(p) = pi d ; ps "force " ; assignment p
	      | procedural_continuous_assignment d Release(p) = pi d ; ps "release " ; lvalue p
	    and procedural_timing_control_statement d (c,s) = pi d ; delay_or_event_control c ; ps " " ; statement_or_null 0 s
	    and range (e1,e2) = ps "[" ; constant_expression e1 ; ps ":" ; constant_expression e2 ; ps "]"
	    and range_or_type _ _ = ()
	    and real_declaration _ _ = ()
	    and realtime_declaration _ _ = ()
	    and reg_declaration d (r,l) = pi d ; ps "reg" ; po range r ; ps " " ; list_of_register_variables l ; psn ";"
	    and register_variable Register(r,None) = name_of_register r
	      | register_variable Register(r,Some(e)) = name_of_register r ; ps "=" ; expression e
	      | register_variable Memory(m,e1,e2) = name_of_memory m ; ps "[" ; constant_expression e1 ; ps ":" ; constant_expression e2 ; ps "]"
	    and scalar_event_expression e = expression e
	    and seq_block d (None,sl) = pi d ; psn "begin" ; pl (statement (d+1)) "" sl ; pi d ; psn "end"
	      | seq_block d (Some(n,b),sl) = pi d ; ps "begin : " ; name_of_block n ; pn() ; pl (block_item_declaration (d+1)) "\n" b ; pn() ; pl (statement (d+1)) "" sl ; pi d ; psn "end"
	    and sequential_entry _ _ = ()
	    and single_identifier i = ps i
	    and source_text d desc = pl (description d) "\n" desc
	    and specify_block _ _ = ()
	    and specify_item _ _ = ()
	    and state _ _ = ()
	    and statement d BlockAssign(s) = blocking_assignment d s ; psn ";"
	      | statement d NonBlockAssign(s) = non_blocking_assignment d s ; psn ";"
	      | statement d ProcContAssign(s) = procedural_continuous_assignment d s ; psn ";"
	      | statement d ProcTimingCtrlStmt(s) = procedural_timing_control_statement d s
	      | statement d CondStmt(s) = conditional_statement d s
	      | statement d CaseStmt(s) = case_statement d s
	      | statement d LoopStmt(s) = loop_statement d s
	      | statement d WaitStmt(s) = wait_statement d s
	      | statement d DisableStmt(s) = disable_statement d s
	      | statement d EventTrigger(s) = event_trigger d s
	      | statement d SeqBlock(s) = seq_block d s
	      | statement d ParBlock(s) = par_block d s
	      | statement d TaskEnable(s) = task_enable d s
	      | statement d SystemTaskEnable(s) = system_task_enable d s
	    and statement_or_null d Some(s) = statement d s
	      | statement_or_null d None = pi d ; psn ";"
	    and strength0 SSupply0 = ps "supply0"
	      | strength0 Strong0 = ps "strong0"
	      | strength0 Pull0 = ps "pull0"
	      | strength0 Weak0 = ps "weak0"
	      | strength0 Highz0 = ps "highz0"
	    and strength1 SSupply1 = ps "supply1"
	      | strength1 Strong1 = ps "strong1"
	      | strength1 Pull1 = ps "pull1"
	      | strength1 Weak1 = ps "weak1"
	      | strength1 Highz1 = ps "highz1"
	    and symbol _ _ = ()
	    and system_identifier i = ps "$" ; single_identifier i
	    and system_task_enable d (n,[]) = pi d ; name_of_system_task n ; psn ";"
	      | system_task_enable d (n,e) = pi d ; name_of_system_task n ; ps "(" ; pl expression ", " e ; psn ");"
	    and table_definition _ _ = ()
	    and table_entries _ _ = ()
	    and task d (n,i,s) = pi d ; ps "task " ; name_of_task n ; psn ";" ; pl (task_item_declaration (d+1)) "" i ; statement_or_null (d+1) s ; pi d ; psn "endtask"
	    and task_enable d (n,[]) = pi d ; name_of_task n ; psn ";"
	      | task_enable d (n,l) = pi d ; name_of_task n ; ps "(" ; pl expression ", " l ; psn ");"
	    and task_item_declaration _ _ = ()
	    and terminal _ _ = ()
	    and time_declaration _ _ = ()
	    and udp_declaration _ _ = ()
	    and udp_initial_statement _ _ = ()
	    and udp_instantiation _ _ = ()
	    and udp_port_declaration _ _ = ()
	    and unary_operator UPlus = ps "+"
	      | unary_operator UMinus = ps "-"
	      | unary_operator ULNot = ps "!"
	      | unary_operator UNot = ps "~"
	      | unary_operator UAnd = ps "&"
	      | unary_operator UNand = ps "~&"
	      | unary_operator UOr = ps "|"
	      | unary_operator UNor = ps "~|"
	      | unary_operator UXor = ps "^"
	      | unary_operator UXnor = ps "~^"
	    and wait_statement d (e,s) = pi d ; ps "wait (" ; expression e ; psn ") " ; statement_or_null (d+1) s ; pn()
	in
	    program 0 pgm
	end
end

(* vim: set ts=8 sw=4 sts=4 smarttab: *)
