structure Verilog_lexer = struct
(* # 1 "./verilog_lexer.nl" *)
 
(*
    lcom -> line comment
    ccom -> c comment
*)

    open Verilog_parser

    exception LexicalError

    val line_no = ref 1
    fun inc_line_no() = line_no:=!line_no+1

    val str_buf = ref ""
    fun reset_str_buf() = ( str_buf:="" )
    fun add_to_str_buf c = ( str_buf := !str_buf ^ c )

    val udp_table = ref false

    fun create_hashtable size init =
	let
	    val table = Hashtbl.create size
	in
	    List.iter (fn (key,data) => Hashtbl.add table key data ) init ;
	    table
	end
    val keyword_table =
	create_hashtable 105 [
	    ("always", T_always),
	    ("and", T_and),
	    ("assign", T_assign),
	    ("begin", T_begin),
	    ("buf", T_buf),
	    ("bufif0", T_bufif0),
	    ("bufif1", T_bufif1),
	    ("case", T_case),
	    ("casex", T_casex),
	    ("casez", T_casez),
	    ("cmos", T_cmos),
	    ("deassign", T_deassign),
	    ("default", T_default),
	    ("defparam", T_defparam),
	    ("disable", T_disable),
	    ("edge", T_edge),
	    ("else", T_else),
	    ("end", T_end),
	    ("endcase", T_endcase),
	    ("endfunction", T_endfunction),
	    ("endmodule", T_endmodule),
	    ("endprimitive", T_endprimitive),
	    ("endspecify", T_endspecify),
	    ("endtable", T_endtable),
	    ("endtask", T_endtask),
	    ("event", T_event),
	    ("for", T_for),
	    ("force", T_force),
	    ("forever", T_forever),
	    ("fork", T_fork),
	    ("function", T_function),
	    ("highz0", T_highz0),
	    ("highz1", T_highz1),
	    ("if", T_if),
	    ("initial", T_initial),
	    ("inout", T_inout),
	    ("input", T_input),
	    ("integer", T_integer),
	    ("join", T_join),
	    ("large", T_large),
	    ("localparam", T_localparam),
	    ("macromodule", T_macromodule),
	    ("medium", T_medium),
	    ("module", T_module),
	    ("nand", T_nand),
	    ("negedge", T_negedge),
	    ("nmos", T_nmos),
	    ("nor", T_nor),
	    ("not", T_not),
	    ("notif0", T_notif0),
	    ("notif1", T_notif1),
	    ("or", T_or),
	    ("output", T_output),
	    ("parameter", T_parameter),
	    ("pmos", T_pmos),
	    ("posedge", T_posedge),
	    ("primitive", T_primitive),
	    ("pull0", T_pull0),
	    ("pull1", T_pull1),
	    ("pulldown", T_pulldown),
	    ("pullup", T_pullup),
	    ("rcmos", T_rcmos),
	    ("real", T_real),
	    ("realtime", T_realtime),
	    ("reg", T_reg),
	    ("release", T_release),
	    ("repeat", T_repeat),
	    ("rnmos", T_rnmos),
	    ("rpmos", T_rpmos),
	    ("rtran", T_rtran),
	    ("rtranif0", T_rtranif0),
	    ("rtranif1", T_rtranif1),
	    ("scalared", T_scalared),
	    ("signed", T_signed),
	    ("small", T_small),
	    ("specify", T_specify),
	    ("specparam", T_specparam),
	    ("strong0", T_strong0),
	    ("strong1", T_strong1),
	    ("supply0", T_supply0),
	    ("supply1", T_supply1),
	    ("table", T_table),
	    ("task", T_task),
	    ("time", T_time),
	    ("tran", T_tran),
	    ("tranif0", T_tranif0),
	    ("tranif1", T_tranif1),
	    ("tri", T_tri),
	    ("tri0", T_tri0),
	    ("tri1", T_tri1),
	    ("triand", T_triand),
	    ("trior", T_trior),
	    ("trireg", T_trireg),
	    ("vectored", T_vectored),
	    ("wait", T_wait),
	    ("wand", T_wand),
	    ("weak0", T_weak0),
	    ("weak1", T_weak1),
	    ("while", T_while),
	    ("wire", T_wire),
	    ("wor", T_wor),
	    ("xnor", T_xnor),
	    ("xor", T_xor)
	]
    val sys_keyword_table =
	create_hashtable 10 [
	    ("hold", T_Shold),
	    ("period", T_Speriod),
	    ("recovery", T_Srecovery),
	    ("setup", T_Ssetup),
	    ("setuphold", T_Ssetuphold),
	    ("skew", T_Sskew),
	    ("width", T_Swidth)
	]

    fun print_token t = if !Arguments.print_token then print_string(t^" ")

    fun error msg =
	    if !Arguments.print_token then print_newline() ;
	    print_string "Line #" ;
	    print_int !line_no ;
	    print_string " : '" ;
	    print_string msg ;
	    print_endline "'" ;
	    raise LexicalError
val lex_tables = Interop.lexing_to_lex_tables(
  (* lex_base = *) 
   "\000\000\001\000\117\000\118\000\002\000\253\255\254\255\255\255\
    \252\255\253\255\254\255\082\000\254\255\194\255\250\255\251\255\
    \069\000\252\255\247\255\095\000\246\255\093\000\155\000\244\255\
    \243\255\070\000\075\000\240\255\076\000\238\255\093\000\191\000\
    \236\255\235\255\076\000\080\000\082\000\231\255\230\255\228\000\
    \229\255\241\000\228\255\013\000\249\255\226\255\021\000\224\255\
    \195\000\206\255\208\255\209\255\212\255\207\255\003\001\059\001\
    \219\255\221\255\085\000\218\255\214\255\222\255\220\255\116\001\
    \048\001\148\001\180\001\212\001\222\001\232\001\025\002\030\002\
    \085\002\111\002\001\002\160\002\049\002\186\002\203\255\205\255\
    \204\255\217\255\109\000\210\255\247\002\087\000\213\255",
  (* lex_backtrk = *) 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\003\000\255\255\255\255\255\255\255\255\
    \007\000\255\255\255\255\061\000\255\255\010\000\061\000\255\255\
    \255\255\013\000\014\000\255\255\016\000\255\255\018\000\060\000\
    \255\255\255\255\021\000\022\000\023\000\255\255\255\255\053\000\
    \255\255\061\000\255\255\028\000\255\255\255\255\030\000\255\255\
    \032\000\255\255\255\255\255\255\255\255\255\255\054\000\053\000\
    \255\255\255\255\039\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\060\000\255\255\255\255\060\000\060\000\255\255\255\255\
    \255\255\255\255\057\000\059\000\056\000\058\000\255\255\255\255\
    \255\255\255\255\044\000\255\255\055\000\040\000\255\255",
  (* lex_default = *) 
   "\013\000\005\000\008\000\008\000\005\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\000\000\000\000\000\000\000\000\
    \255\255\000\000\000\000\255\255\000\000\255\255\255\255\000\000\
    \000\000\255\255\255\255\000\000\255\255\000\000\255\255\255\255\
    \000\000\000\000\255\255\255\255\255\255\000\000\000\000\255\255\
    \000\000\054\000\000\000\255\255\000\000\000\000\255\255\000\000\
    \255\255\000\000\000\000\000\000\000\000\000\000\054\000\255\255\
    \000\000\000\000\255\255\000\000\000\000\000\000\000\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\000\000\000\000\
    \000\000\000\000\255\255\000\000\255\255\255\255\000\000",
  (* lex_trans = *) 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \014\000\014\000\015\000\006\000\006\000\014\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \014\000\016\000\017\000\018\000\019\000\020\000\021\000\022\000\
    \023\000\024\000\025\000\026\000\027\000\028\000\029\000\030\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\032\000\033\000\034\000\035\000\036\000\037\000\
    \038\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\040\000\041\000\042\000\043\000\039\000\
    \044\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\045\000\046\000\047\000\048\000\009\000\
    \009\000\012\000\085\000\082\000\081\000\080\000\078\000\005\000\
    \061\000\062\000\079\000\053\000\012\000\058\000\059\000\056\000\
    \057\000\052\000\060\000\083\000\086\000\000\000\000\000\000\000\
    \010\000\000\000\000\000\000\000\000\000\000\000\000\000\011\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\000\000\000\000\000\000\000\000\084\000\000\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\000\000\000\000\000\000\070\000\000\000\071\000\
    \000\000\000\000\000\000\072\000\000\000\000\000\063\000\000\000\
    \000\000\049\000\073\000\000\000\000\000\064\000\000\000\065\000\
    \065\000\065\000\065\000\065\000\065\000\065\000\065\000\065\000\
    \065\000\255\255\255\255\255\255\000\000\070\000\255\255\071\000\
    \007\000\007\000\007\000\072\000\066\000\000\000\000\000\000\000\
    \055\000\000\000\073\000\255\255\255\255\255\255\000\000\000\000\
    \255\255\255\255\000\000\000\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\065\000\000\000\
    \000\000\050\000\000\000\255\255\066\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\051\000\
    \000\000\000\000\000\000\055\000\000\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\000\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\007\000\007\000\000\000\
    \000\000\000\000\000\000\000\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\000\000\000\000\
    \000\000\000\000\055\000\000\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\070\000\000\000\
    \071\000\000\000\000\000\063\000\072\000\000\000\000\000\000\000\
    \000\000\000\000\064\000\073\000\065\000\065\000\065\000\065\000\
    \065\000\065\000\065\000\065\000\065\000\065\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\070\000\000\000\
    \071\000\066\000\000\000\000\000\072\000\000\000\000\000\067\000\
    \000\000\067\000\000\000\073\000\068\000\068\000\068\000\068\000\
    \068\000\068\000\068\000\068\000\068\000\068\000\000\000\000\000\
    \000\000\255\255\000\000\065\000\000\000\000\000\000\000\000\000\
    \000\000\066\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\255\255\068\000\068\000\068\000\068\000\
    \068\000\068\000\068\000\068\000\068\000\068\000\068\000\068\000\
    \068\000\068\000\068\000\068\000\068\000\068\000\068\000\068\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\070\000\000\000\000\000\000\000\000\000\071\000\
    \000\000\000\000\000\000\000\000\000\000\066\000\000\000\000\000\
    \000\000\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \074\000\070\000\000\000\000\000\000\000\068\000\071\000\000\000\
    \074\000\000\000\000\000\000\000\000\000\000\000\000\000\069\000\
    \000\000\077\000\077\000\000\000\000\000\066\000\076\000\076\000\
    \076\000\076\000\076\000\076\000\076\000\076\000\076\000\076\000\
    \077\000\074\000\000\000\074\000\000\000\000\000\072\000\000\000\
    \074\000\076\000\076\000\076\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\076\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\077\000\000\000\077\000\000\000\072\000\000\000\000\000\
    \073\000\074\000\000\000\074\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\075\000\075\000\075\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\073\000\
    \076\000\077\000\000\000\077\000\075\000\000\000\075\000\075\000\
    \075\000\075\000\075\000\075\000\000\000\000\000\000\000\074\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\074\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\075\000\074\000\075\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\075\000\075\000\
    \075\000\075\000\075\000\075\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\074\000\
    \000\000\074\000\000\000\000\000\000\000\075\000\000\000\075\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\075\000\000\000\000\000\000\000\000\000\000\000\075\000\
    \000\000\075\000\075\000\075\000\075\000\075\000\075\000\074\000\
    \000\000\074\000\077\000\077\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \075\000\077\000\075\000\000\000\000\000\000\000\000\000\075\000\
    \000\000\075\000\075\000\075\000\075\000\075\000\075\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\077\000\000\000\077\000\000\000\000\000\000\000\
    \075\000\077\000\075\000\084\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\000\000\077\000\000\000\077\000\000\000\000\000\000\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\000\000\000\000\000\000\000\000\084\000\000\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    ",
  (* lex_check = *) 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\001\000\004\000\000\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\002\000\
    \003\000\011\000\016\000\021\000\025\000\026\000\028\000\030\000\
    \034\000\034\000\028\000\043\000\030\000\035\000\035\000\036\000\
    \036\000\046\000\058\000\082\000\085\000\255\255\255\255\255\255\
    \003\000\255\255\255\255\255\255\255\255\255\255\255\255\002\000\
    \019\000\019\000\019\000\019\000\019\000\019\000\019\000\019\000\
    \019\000\019\000\019\000\019\000\019\000\019\000\019\000\019\000\
    \019\000\019\000\019\000\019\000\019\000\019\000\019\000\019\000\
    \019\000\019\000\255\255\255\255\255\255\255\255\019\000\255\255\
    \019\000\019\000\019\000\019\000\019\000\019\000\019\000\019\000\
    \019\000\019\000\019\000\019\000\019\000\019\000\019\000\019\000\
    \019\000\019\000\019\000\019\000\019\000\019\000\019\000\019\000\
    \019\000\019\000\255\255\255\255\255\255\022\000\255\255\022\000\
    \255\255\255\255\255\255\022\000\255\255\255\255\031\000\255\255\
    \255\255\048\000\022\000\255\255\255\255\031\000\255\255\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\041\000\041\000\041\000\255\255\022\000\041\000\022\000\
    \000\000\001\000\004\000\022\000\031\000\255\255\255\255\255\255\
    \039\000\255\255\022\000\054\000\054\000\054\000\255\255\255\255\
    \054\000\041\000\255\255\255\255\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\031\000\255\255\
    \255\255\048\000\255\255\054\000\031\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\048\000\
    \255\255\255\255\255\255\039\000\255\255\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\039\000\
    \039\000\039\000\039\000\039\000\039\000\039\000\039\000\055\000\
    \064\000\064\000\064\000\064\000\064\000\064\000\064\000\064\000\
    \064\000\064\000\255\255\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\002\000\003\000\255\255\
    \255\255\255\255\255\255\255\255\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\255\255\255\255\
    \255\255\255\255\055\000\255\255\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\055\000\055\000\
    \055\000\055\000\055\000\055\000\055\000\055\000\063\000\255\255\
    \063\000\255\255\255\255\065\000\063\000\255\255\255\255\255\255\
    \255\255\255\255\065\000\063\000\065\000\065\000\065\000\065\000\
    \065\000\065\000\065\000\065\000\065\000\065\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\063\000\255\255\
    \063\000\065\000\255\255\255\255\063\000\255\255\255\255\066\000\
    \255\255\066\000\255\255\063\000\066\000\066\000\066\000\066\000\
    \066\000\066\000\066\000\066\000\066\000\066\000\255\255\255\255\
    \255\255\041\000\255\255\065\000\255\255\255\255\255\255\255\255\
    \255\255\065\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\054\000\067\000\067\000\067\000\067\000\
    \067\000\067\000\067\000\067\000\067\000\067\000\068\000\068\000\
    \068\000\068\000\068\000\068\000\068\000\068\000\068\000\068\000\
    \069\000\069\000\069\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\070\000\255\255\255\255\255\255\255\255\071\000\
    \255\255\255\255\255\255\255\255\255\255\069\000\255\255\255\255\
    \255\255\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \074\000\070\000\255\255\255\255\255\255\068\000\071\000\255\255\
    \074\000\255\255\255\255\255\255\255\255\255\255\255\255\069\000\
    \255\255\070\000\070\000\255\255\255\255\069\000\071\000\071\000\
    \071\000\071\000\071\000\071\000\071\000\071\000\071\000\071\000\
    \070\000\074\000\255\255\074\000\255\255\255\255\072\000\255\255\
    \074\000\076\000\076\000\076\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\076\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\070\000\255\255\070\000\255\255\072\000\255\255\255\255\
    \073\000\074\000\255\255\074\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\072\000\072\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\072\000\073\000\
    \076\000\070\000\255\255\070\000\072\000\255\255\072\000\072\000\
    \072\000\072\000\072\000\072\000\255\255\255\255\255\255\073\000\
    \073\000\073\000\073\000\073\000\073\000\073\000\073\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\072\000\073\000\072\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\072\000\072\000\
    \072\000\072\000\072\000\072\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\073\000\
    \255\255\073\000\255\255\255\255\255\255\072\000\255\255\072\000\
    \075\000\075\000\075\000\075\000\075\000\075\000\075\000\075\000\
    \075\000\075\000\255\255\255\255\255\255\255\255\255\255\075\000\
    \255\255\075\000\075\000\075\000\075\000\075\000\075\000\073\000\
    \255\255\073\000\077\000\077\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \075\000\077\000\075\000\255\255\255\255\255\255\255\255\075\000\
    \255\255\075\000\075\000\075\000\075\000\075\000\075\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\077\000\255\255\077\000\255\255\255\255\255\255\
    \075\000\077\000\075\000\084\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\255\255\077\000\255\255\077\000\255\255\255\255\255\255\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\255\255\255\255\255\255\255\255\084\000\255\255\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\084\000\084\000\084\000\084\000\084\000\084\000\
    \084\000\084\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    "
)

fun main lexbuf = n_lex_main_rec lexbuf 0
and n_lex_main_rec lexbuf state =
 case (Lexing.engine lex_tables state lexbuf) of
    0 => (
(* # 174 "./verilog_lexer.nl" *)
            T_EOF )
  | 1 => (
(* # 175 "./verilog_lexer.nl" *)
             lcom lexbuf )
  | 2 => (
(* # 176 "./verilog_lexer.nl" *)
             ccom lexbuf )
  | 3 => (
(* # 177 "./verilog_lexer.nl" *)
            reset_str_buf() ; string lexbuf )
  | 4 => (
(* # 178 "./verilog_lexer.nl" *)
             inc_line_no() ; main lexbuf )
  | 5 => (
(* # 179 "./verilog_lexer.nl" *)
                   main lexbuf )
  | 6 => (
(* # 180 "./verilog_lexer.nl" *)
            directive lexbuf )
  | 7 => (
(* # 181 "./verilog_lexer.nl" *)
            print_token("Bang") ; T_BANG )
  | 8 => (
(* # 182 "./verilog_lexer.nl" *)
            print_token("Hash") ; T_HASH )
  | 9 => (
(* # 183 "./verilog_lexer.nl" *)
            print_token("Percent") ; T_PERCENT )
  | 10 => (
(* # 184 "./verilog_lexer.nl" *)
            print_token("Amp") ; T_AMP )
  | 11 => (
(* # 185 "./verilog_lexer.nl" *)
            print_token("Lparen") ; T_LPAREN )
  | 12 => (
(* # 186 "./verilog_lexer.nl" *)
            print_token("Rparen") ; T_RPAREN )
  | 13 => (
(* # 187 "./verilog_lexer.nl" *)
            print_token("Asterisk") ; T_ASTERISK )
  | 14 => (
(* # 188 "./verilog_lexer.nl" *)
            print_token("Plus") ; T_PLUS )
  | 15 => (
(* # 189 "./verilog_lexer.nl" *)
            print_token("Comma") ; T_COMMA )
  | 16 => (
(* # 190 "./verilog_lexer.nl" *)
            print_token("Minus") ; T_MINUS )
  | 17 => (
(* # 191 "./verilog_lexer.nl" *)
            print_token("Dot") ; T_DOT )
  | 18 => (
(* # 192 "./verilog_lexer.nl" *)
            print_token("Slash") ; T_SLASH )
  | 19 => (
(* # 193 "./verilog_lexer.nl" *)
            print_token("Colon") ; T_COLON )
  | 20 => (
(* # 194 "./verilog_lexer.nl" *)
            print_token("Semicolon") ; T_SEMICOLON )
  | 21 => (
(* # 195 "./verilog_lexer.nl" *)
            print_token("Less") ; T_LESS )
  | 22 => (
(* # 196 "./verilog_lexer.nl" *)
            print_token("Equal") ; T_EQUAL )
  | 23 => (
(* # 197 "./verilog_lexer.nl" *)
            print_token("Greater") ; T_GREATER )
  | 24 => (
(* # 198 "./verilog_lexer.nl" *)
            print_token("Qmark") ; T_QMARK )
  | 25 => (
(* # 199 "./verilog_lexer.nl" *)
            print_token("At") ; T_AT )
  | 26 => (
(* # 200 "./verilog_lexer.nl" *)
            print_token("Lbracket") ; T_LBRACKET )
  | 27 => (
(* # 201 "./verilog_lexer.nl" *)
            print_token("Rbracket") ; T_RBRACKET )
  | 28 => (
(* # 202 "./verilog_lexer.nl" *)
            print_token("Caccent") ; T_CACCENT )
  | 29 => (
(* # 203 "./verilog_lexer.nl" *)
            print_token("Lbrace") ; T_LBRACE )
  | 30 => (
(* # 204 "./verilog_lexer.nl" *)
            print_token("Vline") ; T_VLINE )
  | 31 => (
(* # 205 "./verilog_lexer.nl" *)
            print_token("Rbrace") ; T_RBRACE )
  | 32 => (
(* # 206 "./verilog_lexer.nl" *)
            print_token("Tilde") ; T_TILDE )
  | 33 => (
(* # 207 "./verilog_lexer.nl" *)
             print_token("Lless") ; T_LLESS )
  | 34 => (
(* # 208 "./verilog_lexer.nl" *)
             print_token("Ggreater") ; T_GGREATER )
  | 35 => (
(* # 209 "./verilog_lexer.nl" *)
             print_token("Lequal") ; T_LEQUAL )
  | 36 => (
(* # 210 "./verilog_lexer.nl" *)
             print_token("Gequal") ; T_GEQUAL )
  | 37 => (
(* # 211 "./verilog_lexer.nl" *)
             print_token("Egreater") ; T_EGREATER )
  | 38 => (
(* # 212 "./verilog_lexer.nl" *)
             print_token("Agreater") ; T_AGREATER )
  | 39 => (
(* # 213 "./verilog_lexer.nl" *)
             print_token("Eequal") ; T_EEQUAL )
  | 40 => (
(* # 214 "./verilog_lexer.nl" *)
             print_token("Bequal") ; T_BEQUAL )
  | 41 => (
(* # 215 "./verilog_lexer.nl" *)
              print_token("Eeequal") ; T_EEEQUAL )
  | 42 => (
(* # 216 "./verilog_lexer.nl" *)
              print_token("Beequal") ; T_BEEQUAL )
  | 43 => (
(* # 217 "./verilog_lexer.nl" *)
             print_token("Vvline") ; T_VVLINE )
  | 44 => (
(* # 218 "./verilog_lexer.nl" *)
             print_token("Aamp") ; T_AAMP )
  | 45 => (
(* # 219 "./verilog_lexer.nl" *)
              print_token("Aaamp") ; T_AAAMP )
  | 46 => (
(* # 220 "./verilog_lexer.nl" *)
             print_token("Tvline") ; T_TVLINE )
  | 47 => (
(* # 221 "./verilog_lexer.nl" *)
             print_token("Tcaccent") ; T_TCACCENT )
  | 48 => (
(* # 222 "./verilog_lexer.nl" *)
             print_token("Ctilde") ; T_CTILDE )
  | 49 => (
(* # 223 "./verilog_lexer.nl" *)
             print_token("Tamp") ; T_TAMP )
  | 50 => (
(* # 224 "./verilog_lexer.nl" *)
             print_token("Mgreater") ; T_MGREATER )
  | 51 => (
(* # 225 "./verilog_lexer.nl" *)
             print_token("Pcolon") ; T_PCOLON )
  | 52 => (
(* # 226 "./verilog_lexer.nl" *)
             print_token("Mcolon") ; T_MCOLON )
  | 53 => (
(* # 228 "./verilog_lexer.nl" *)
  
	    let
		val id = Lexing.lexeme lexbuf
		val token = Hashtbl.find keyword_table id
		    handle Not_found => T_ID id
	    in
		print_token id ;
		if !udp_table
		then
		    if token=T_endtable then ( udp_table:=false ; token )
		    else
			case id
			of "x" => T_SymbolX
			 | "X" => T_SymbolX
			 | "b" => T_SymbolB
			 | "B" => T_SymbolB
			 | "r" => T_SymbolR
			 | "R" => T_SymbolR
			 | "f" => T_SymbolF
			 | "F" => T_SymbolF
			 | "p" => T_SymbolP
			 | "P" => T_SymbolP
			 | "n" => T_SymbolN
			 | "N" => T_SymbolN
			 | _ => error("Not UDP Symbol")
		else
		    if token=T_table then ( udp_table:=true ; token )
		    else token
	    end
	)
  | 54 => (
(* # 259 "./verilog_lexer.nl" *)
  
	    let
		val id = Lexing.lexeme lexbuf
		val id2 = String.sub id 1 ((String.length id)-1)
	    in
		print_token id2 ;
		T_ID id2
	    end
	)
  | 55 => (
(* # 269 "./verilog_lexer.nl" *)
  
	    let
		val id = Lexing.lexeme lexbuf
		val id2 = String.sub id 1 ((String.length id)-1)
	    in
		print_token id2 ;
		Hashtbl.find sys_keyword_table id2
		    handle Not_found => T_SID id2
	    end
	)
  | 56 => (
(* # 280 "./verilog_lexer.nl" *)
   print_token("Dec") ; T_NUM(Lexing.lexeme lexbuf) )
  | 57 => (
(* # 282 "./verilog_lexer.nl" *)
   print_token("Oct") ; T_NUM(Lexing.lexeme lexbuf) )
  | 58 => (
(* # 284 "./verilog_lexer.nl" *)
   print_token("Bin") ; T_NUM(Lexing.lexeme lexbuf) )
  | 59 => (
(* # 286 "./verilog_lexer.nl" *)
   print_token("Hex") ; T_NUM(Lexing.lexeme lexbuf) )
  | 60 => (
(* # 288 "./verilog_lexer.nl" *)
  
	    let
		val num = Lexing.lexeme lexbuf
	    in
		if !udp_table
		then
		    case num
		    of "0" => print_token("0") ; T_Symbol0
		     | "1" => print_token("1") ; T_Symbol1
		     | _ => error("Not UDP Symbol")
		else ( print_token("Num") ; T_NUM(num) )
	    end
	)
  | 61 => (
(* # 301 "./verilog_lexer.nl" *)
          error(Lexing.lexeme lexbuf) )
  | n => (Interop.lexing_field_refill_buff lexbuf) lexbuf; n_lex_main_rec lexbuf n

and lcom lexbuf = n_lex_lcom_rec lexbuf 1
and n_lex_lcom_rec lexbuf state =
 case (Lexing.engine lex_tables state lexbuf) of
    0 => (
(* # 304 "./verilog_lexer.nl" *)
            T_EOF )
  | 1 => (
(* # 305 "./verilog_lexer.nl" *)
             inc_line_no() ; print_token("LineComment") ; main lexbuf )
  | 2 => (
(* # 306 "./verilog_lexer.nl" *)
          lcom lexbuf )
  | n => (Interop.lexing_field_refill_buff lexbuf) lexbuf; n_lex_lcom_rec lexbuf n

and ccom lexbuf = n_lex_ccom_rec lexbuf 2
and n_lex_ccom_rec lexbuf state =
 case (Lexing.engine lex_tables state lexbuf) of
    0 => (
(* # 309 "./verilog_lexer.nl" *)
            error("Missing close mark of comment") )
  | 1 => (
(* # 310 "./verilog_lexer.nl" *)
             print_token("CComment") ; main lexbuf )
  | 2 => (
(* # 311 "./verilog_lexer.nl" *)
             inc_line_no() ; ccom lexbuf )
  | 3 => (
(* # 312 "./verilog_lexer.nl" *)
          ccom lexbuf )
  | n => (Interop.lexing_field_refill_buff lexbuf) lexbuf; n_lex_ccom_rec lexbuf n

and string lexbuf = n_lex_string_rec lexbuf 3
and n_lex_string_rec lexbuf state =
 case (Lexing.engine lex_tables state lexbuf) of
    0 => (
(* # 315 "./verilog_lexer.nl" *)
            error("Missing close quote of comment") )
  | 1 => (
(* # 316 "./verilog_lexer.nl" *)
            print_token("String") ; T_STR(!str_buf) )
  | 2 => (
(* # 317 "./verilog_lexer.nl" *)
             error("Missing close quote of comment") )
  | 3 => (
(* # 318 "./verilog_lexer.nl" *)
          add_to_str_buf(Lexing.lexeme lexbuf) ; string lexbuf )
  | n => (Interop.lexing_field_refill_buff lexbuf) lexbuf; n_lex_string_rec lexbuf n

and directive lexbuf = n_lex_directive_rec lexbuf 4
and n_lex_directive_rec lexbuf state =
 case (Lexing.engine lex_tables state lexbuf) of
    0 => (
(* # 321 "./verilog_lexer.nl" *)
            T_EOF )
  | 1 => (
(* # 322 "./verilog_lexer.nl" *)
             print_token("Directive") ; main lexbuf )
  | 2 => (
(* # 323 "./verilog_lexer.nl" *)
          directive lexbuf )
  | n => (Interop.lexing_field_refill_buff lexbuf) lexbuf; n_lex_directive_rec lexbuf n


end
