{
(*
    lcom -> line comment
    ccom -> c comment
*)

    open Verilog_parser

    exception LexicalError

    val line_no = ref 1
    fun inc_line_no() = line_no:=!line_no+1

    val str_buf = ref ""
    fun reset_str_buf() = ( str_buf:="" )
    fun add_to_str_buf c = ( str_buf := !str_buf ^ c )

    val udp_table = ref false

    fun create_hashtable size init =
	let
	    val table = Hashtbl.create size
	in
	    List.iter (fn (key,data) => Hashtbl.add table key data ) init ;
	    table
	end
    val keyword_table =
	create_hashtable 105 [
	    ("always", T_always),
	    ("and", T_and),
	    ("assign", T_assign),
	    ("begin", T_begin),
	    ("buf", T_buf),
	    ("bufif0", T_bufif0),
	    ("bufif1", T_bufif1),
	    ("case", T_case),
	    ("casex", T_casex),
	    ("casez", T_casez),
	    ("cmos", T_cmos),
	    ("deassign", T_deassign),
	    ("default", T_default),
	    ("defparam", T_defparam),
	    ("disable", T_disable),
	    ("edge", T_edge),
	    ("else", T_else),
	    ("end", T_end),
	    ("endcase", T_endcase),
	    ("endfunction", T_endfunction),
	    ("endmodule", T_endmodule),
	    ("endprimitive", T_endprimitive),
	    ("endspecify", T_endspecify),
	    ("endtable", T_endtable),
	    ("endtask", T_endtask),
	    ("event", T_event),
	    ("for", T_for),
	    ("force", T_force),
	    ("forever", T_forever),
	    ("fork", T_fork),
	    ("function", T_function),
	    ("highz0", T_highz0),
	    ("highz1", T_highz1),
	    ("if", T_if),
	    ("initial", T_initial),
	    ("inout", T_inout),
	    ("input", T_input),
	    ("integer", T_integer),
	    ("join", T_join),
	    ("large", T_large),
	    ("localparam", T_localparam),
	    ("macromodule", T_macromodule),
	    ("medium", T_medium),
	    ("module", T_module),
	    ("nand", T_nand),
	    ("negedge", T_negedge),
	    ("nmos", T_nmos),
	    ("nor", T_nor),
	    ("not", T_not),
	    ("notif0", T_notif0),
	    ("notif1", T_notif1),
	    ("or", T_or),
	    ("output", T_output),
	    ("parameter", T_parameter),
	    ("pmos", T_pmos),
	    ("posedge", T_posedge),
	    ("primitive", T_primitive),
	    ("pull0", T_pull0),
	    ("pull1", T_pull1),
	    ("pulldown", T_pulldown),
	    ("pullup", T_pullup),
	    ("rcmos", T_rcmos),
	    ("real", T_real),
	    ("realtime", T_realtime),
	    ("reg", T_reg),
	    ("release", T_release),
	    ("repeat", T_repeat),
	    ("rnmos", T_rnmos),
	    ("rpmos", T_rpmos),
	    ("rtran", T_rtran),
	    ("rtranif0", T_rtranif0),
	    ("rtranif1", T_rtranif1),
	    ("scalared", T_scalared),
	    ("signed", T_signed),
	    ("small", T_small),
	    ("specify", T_specify),
	    ("specparam", T_specparam),
	    ("strong0", T_strong0),
	    ("strong1", T_strong1),
	    ("supply0", T_supply0),
	    ("supply1", T_supply1),
	    ("table", T_table),
	    ("task", T_task),
	    ("time", T_time),
	    ("tran", T_tran),
	    ("tranif0", T_tranif0),
	    ("tranif1", T_tranif1),
	    ("tri", T_tri),
	    ("tri0", T_tri0),
	    ("tri1", T_tri1),
	    ("triand", T_triand),
	    ("trior", T_trior),
	    ("trireg", T_trireg),
	    ("vectored", T_vectored),
	    ("wait", T_wait),
	    ("wand", T_wand),
	    ("weak0", T_weak0),
	    ("weak1", T_weak1),
	    ("while", T_while),
	    ("wire", T_wire),
	    ("wor", T_wor),
	    ("xnor", T_xnor),
	    ("xor", T_xor)
	]
    val sys_keyword_table =
	create_hashtable 10 [
	    ("hold", T_Shold),
	    ("period", T_Speriod),
	    ("recovery", T_Srecovery),
	    ("setup", T_Ssetup),
	    ("setuphold", T_Ssetuphold),
	    ("skew", T_Sskew),
	    ("width", T_Swidth)
	]

    fun print_token t = if !Arguments.print_token then print_string(t^" ")

    fun error msg =
	    if !Arguments.print_token then print_newline() ;
	    print_string "Line #" ;
	    print_int !line_no ;
	    print_string " : '" ;
	    print_string msg ;
	    print_endline "'" ;
	    raise LexicalError
}

val whitespace = [ ' ' '\t' '\b' '\r' ]
val upper = ['A'-'Z']
val lower = ['a'-'z']
val letter = ( upper | lower | '_' )

val sign = [ '+' '-' ]
val decdigit = ['0'-'9']
val octdigit = [ '0'-'7' 'x' 'X' 'z' 'Z' '?' ]
val bindigit = [ '0' '1' 'x' 'X' 'z' 'Z' '?' ]
val hexdigit = [ '0'-'9' 'a'-'f' 'A'-'F' 'x' 'X' 'z' 'Z' '?' ]

val decnum = decdigit (decdigit|'_')*
val octnum = octdigit (octdigit|'_')*
val binnum = bindigit (bindigit|'_')*
val hexnum = hexdigit (hexdigit|'_')*
val size = decnum

rule main =
parse eof { T_EOF }
    | "//" { lcom lexbuf }
    | "/*" { ccom lexbuf }
    | '"' { reset_str_buf() ; string lexbuf }
    | '\n' { inc_line_no() ; main lexbuf }
    | whitespace { main lexbuf }
    | '`' { directive lexbuf }
    | '!' { print_token("Bang") ; T_BANG }
    | '#' { print_token("Hash") ; T_HASH }
    | '%' { print_token("Percent") ; T_PERCENT }
    | '&' { print_token("Amp") ; T_AMP }
    | '(' { print_token("Lparen") ; T_LPAREN }
    | ')' { print_token("Rparen") ; T_RPAREN }
    | '*' { print_token("Asterisk") ; T_ASTERISK }
    | '+' { print_token("Plus") ; T_PLUS }
    | ',' { print_token("Comma") ; T_COMMA }
    | '-' { print_token("Minus") ; T_MINUS }
    | '.' { print_token("Dot") ; T_DOT }
    | '/' { print_token("Slash") ; T_SLASH }
    | ':' { print_token("Colon") ; T_COLON }
    | ';' { print_token("Semicolon") ; T_SEMICOLON }
    | '<' { print_token("Less") ; T_LESS }
    | '=' { print_token("Equal") ; T_EQUAL }
    | '>' { print_token("Greater") ; T_GREATER }
    | '?' { print_token("Qmark") ; T_QMARK }
    | '@' { print_token("At") ; T_AT }
    | '[' { print_token("Lbracket") ; T_LBRACKET }
    | ']' { print_token("Rbracket") ; T_RBRACKET }
    | '^' { print_token("Caccent") ; T_CACCENT }
    | '{' { print_token("Lbrace") ; T_LBRACE }
    | '|' { print_token("Vline") ; T_VLINE }
    | '}' { print_token("Rbrace") ; T_RBRACE }
    | '~' { print_token("Tilde") ; T_TILDE }
    | "<<" { print_token("Lless") ; T_LLESS }
    | ">>" { print_token("Ggreater") ; T_GGREATER }
    | "<=" { print_token("Lequal") ; T_LEQUAL }
    | ">=" { print_token("Gequal") ; T_GEQUAL }
    | "=>" { print_token("Egreater") ; T_EGREATER }
    | "*>" { print_token("Agreater") ; T_AGREATER }
    | "==" { print_token("Eequal") ; T_EEQUAL }
    | "!=" { print_token("Bequal") ; T_BEQUAL }
    | "===" { print_token("Eeequal") ; T_EEEQUAL }
    | "!==" { print_token("Beequal") ; T_BEEQUAL }
    | "||" { print_token("Vvline") ; T_VVLINE }
    | "&&" { print_token("Aamp") ; T_AAMP }
    | "&&&" { print_token("Aaamp") ; T_AAAMP }
    | "~|" { print_token("Tvline") ; T_TVLINE }
    | "~^" { print_token("Tcaccent") ; T_TCACCENT }
    | "^~" { print_token("Ctilde") ; T_CTILDE }
    | "~&" { print_token("Tamp") ; T_TAMP }
    | "->" { print_token("Mgreater") ; T_MGREATER }
    | "+:" { print_token("Pcolon") ; T_PCOLON }
    | "-:" { print_token("Mcolon") ; T_MCOLON }
    | letter (letter|decdigit|'$')*
	{
	    let
		val id = Lexing.lexeme lexbuf
		val token = Hashtbl.find keyword_table id
		    handle Not_found => T_ID id
	    in
		print_token id ;
		if !udp_table
		then
		    if token=T_endtable then ( udp_table:=false ; token )
		    else
			case id
			of "x" => T_SymbolX
			 | "X" => T_SymbolX
			 | "b" => T_SymbolB
			 | "B" => T_SymbolB
			 | "r" => T_SymbolR
			 | "R" => T_SymbolR
			 | "f" => T_SymbolF
			 | "F" => T_SymbolF
			 | "p" => T_SymbolP
			 | "P" => T_SymbolP
			 | "n" => T_SymbolN
			 | "N" => T_SymbolN
			 | _ => error("Not UDP Symbol")
		else
		    if token=T_table then ( udp_table:=true ; token )
		    else token
	    end
	}
    | '\\' [^' ''\t''\b''\r''\n']+
	{
	    let
		val id = Lexing.lexeme lexbuf
		val id2 = String.sub id 1 ((String.length id)-1)
	    in
		print_token id2 ;
		T_ID id2
	    end
	}
    | '$' letter (letter|decdigit|'$')*
	{
	    let
		val id = Lexing.lexeme lexbuf
		val id2 = String.sub id 1 ((String.length id)-1)
	    in
		print_token id2 ;
		Hashtbl.find sys_keyword_table id2
		    handle Not_found => T_SID id2
	    end
	}
    | size? '\'' ['d''D'] [' ''\t']* decnum
	{ print_token("Dec") ; T_NUM(Lexing.lexeme lexbuf) }
    | size? '\'' ['o''O'] [' ''\t']* octnum
	{ print_token("Oct") ; T_NUM(Lexing.lexeme lexbuf) }
    | size? '\'' ['b''B'] [' ''\t']* binnum
	{ print_token("Bin") ; T_NUM(Lexing.lexeme lexbuf) }
    | size? '\'' ['h''H'] [' ''\t']* hexnum
	{ print_token("Hex") ; T_NUM(Lexing.lexeme lexbuf) }
    | decnum ('.' decnum)? (['e''E'] sign? decnum)?
	{
	    let
		val num = Lexing.lexeme lexbuf
	    in
		if !udp_table
		then
		    case num
		    of "0" => print_token("0") ; T_Symbol0
		     | "1" => print_token("1") ; T_Symbol1
		     | _ => error("Not UDP Symbol")
		else ( print_token("Num") ; T_NUM(num) )
	    end
	}
    | _ { error(Lexing.lexeme lexbuf) }

 and lcom =
parse eof { T_EOF }
    | '\n' { inc_line_no() ; print_token("LineComment") ; main lexbuf }
    | _ { lcom lexbuf }

 and ccom =
parse eof { error("Missing close mark of comment") }
    | "*/" { print_token("CComment") ; main lexbuf }
    | '\n' { inc_line_no() ; ccom lexbuf }
    | _ { ccom lexbuf }

 and string =
parse eof { error("Missing close quote of comment") }
    | '"' { print_token("String") ; T_STR(!str_buf) }
    | '\n' { error("Missing close quote of comment") }
    | _ { add_to_str_buf(Lexing.lexeme lexbuf) ; string lexbuf }

 and directive =
parse eof { T_EOF }
    | '\n' { print_token("Directive") ; main lexbuf }
    | _ { directive lexbuf }

(* vim: set ts=8 sw=4 sts=4 smarttab: *)
