/*
	http://www.unicode.org/charts/PDF/U0000.pdf
	!	bang		#	hash
	$	dollar(n)	%	percent
	&	amp		(	lparen
	)	rparen		*	asterisk
	+	plus		,	comma
	-	minus		.	dot
	/	slash		:	colon
	;	semicolon	<	less
	=	equal		>	greater
	?	qmark		@	at
	[	lbracket	\	bslash(n)
	]	rbracket	^	caccent
	_	lowline(n)	`	gaccent(n)
	{	lbrace		|	vline
	}	rbrace		~	tilde

	<<	lless		>>	ggreater
	<=	lequal		>=	gequal
	=>	egreater	*>	agreater
	==	eequal		!=	bequal
	===	eeequal		!==	beequal
	||	vvline		&&	aamp
	&&&	aaamp		~|	tvline
	~^	tcaccent	^~	ctilde
	~&	tamp		->	mgreater
	+:	pcolon		-:	mcolon

	T_ID -> identifier
	T_SID -> system identifier
	T_STR -> string
	T_NUM -> number

	list1 -> + in regular expression
	list -> * in regular expression
	opt -> ? in regular expression
*/

%{
    open Verilog_ast
%}

%token <string> T_ID T_SID T_STR T_NUM

%token T_BANG T_HASH T_PERCENT T_AMP T_LPAREN T_RPAREN T_ASTERISK
%token T_PLUS T_COMMA T_MINUS T_DOT T_SLASH T_COLON T_SEMICOLON
%token T_LESS T_EQUAL T_GREATER T_QMARK T_AT T_LBRACKET T_RBRACKET
%token T_CACCENT T_LBRACE T_VLINE T_RBRACE T_TILDE

%token T_LLESS T_GGREATER T_LEQUAL T_GEQUAL T_EGREATER T_AGREATER
%token T_EEQUAL T_BEQUAL T_EEEQUAL T_BEEQUAL T_VVLINE T_AAMP T_AAAMP
%token T_TVLINE T_TCACCENT T_CTILDE T_TAMP T_MGREATER T_PCOLON T_MCOLON

%token T_SymbolX T_SymbolB T_SymbolR T_SymbolF
%token T_SymbolP T_SymbolN T_Symbol0 T_Symbol1

%token T_always T_and T_assign T_begin T_buf T_bufif0 T_bufif1 T_case
%token T_casex T_casez T_cmos T_deassign T_default T_defparam T_disable
%token T_edge T_else T_end T_endcase T_endfunction T_endmodule
%token T_endprimitive T_endspecify T_endtable T_endtask T_event T_for
%token T_force T_forever T_fork T_function T_highz0 T_highz1 T_if
%token T_initial T_inout T_input T_integer T_join T_large T_localparam
%token T_macromodule T_medium T_module T_nand T_negedge T_nmos T_nor
%token T_not T_notif0 T_notif1 T_or T_output T_parameter T_pmos T_posedge
%token T_primitive T_pull0 T_pull1 T_pulldown T_pullup T_rcmos T_real
%token T_realtime T_reg T_release T_repeat T_rnmos T_rpmos T_rtran
%token T_rtranif0 T_rtranif1 T_scalared T_signed T_small T_specify
%token T_specparam T_strong0 T_strong1 T_supply0 T_supply1 T_table
%token T_task T_time T_tran T_tranif0 T_tranif1 T_tri T_tri0 T_tri1
%token T_triand T_trior T_trireg T_vectored T_wait T_wand T_weak0
%token T_weak1 T_while T_wire T_wor T_xnor T_xor

%token T_Shold T_Speriod T_Srecovery T_Ssetup T_Ssetuphold T_Sskew T_Swidth

%token T_EOF

%start main
%type <Verilog_ast.program> main

%nonassoc T_else
%right T_QMARK T_COLON
%left T_VVLINE
%left T_AAMP
%left T_VLINE T_TVLINE
%left T_AMP T_TAMP T_CACCENT T_TCACCENT T_CTILDE
%left T_EEQUAL T_BEQUAL T_EEEQUAL T_BEEQUAL
%left T_LESS T_LEQUAL T_GREATER T_GEQUAL
%left T_LLESS T_GGREATER
%left T_PLUS T_MINUS
%left T_ASTERISK T_SLASH T_PERCENT
%left UNARY_PREC

%%

main
    : source_text T_EOF { $1 }

always_statement
    : T_always statement { $2 }

assignment
    : lvalue T_EQUAL expression { ($1,$3) }

assignment_list1
    : assignment { [$1] }
    | assignment T_COMMA assignment_list1 { $1::$3 }

block_item_declaration
    : parameter_declaration { ParamDecl($1) }
    | reg_declaration { RegDecl($1) }
    | integer_declaration { IntegerDecl($1) }
    | real_declaration { RealDecl($1) }
    | time_declaration { TimeDecl($1) }
    | realtime_declaration { RealtimeDecl($1) }
    | event_declaration { EventDecl($1) }

block_item_declaration_list
    : { [] }
    | block_item_declaration block_item_declaration_list { $1::$2 }

blocking_assignment
    : lvalue T_EQUAL delay_or_event_control_opt expression { ($1,$3,$4) }

case_item
    : expression_list1 T_COLON statement_or_null { (Some($1),$3) }
    | T_default T_COLON statement_or_null { (None,$3) }
    | T_default statement_or_null { (None,$2) }

case_item_list1
    : case_item { [$1] }
    | case_item case_item_list1 { $1::$2 }

case_statement
    : case_type T_LPAREN expression T_RPAREN case_item_list1 T_endcase
	{ ($1,$3,$5) }

case_type
    : T_case { Case }
    | T_casez { CaseZ }
    | T_casex { CaseX }

charge_strength
    : T_LPAREN T_small T_RPAREN { Small }
    | T_LPAREN T_medium T_RPAREN { Medium }
    | T_LPAREN T_large T_RPAREN { Large }

charge_strength_opt
    : { None }
    | charge_strength { Some($1) }

combinational_entry
    : level_input_list T_COLON output_symbol T_SEMICOLON { ($1,$3) }

combinational_entry_list1
    : combinational_entry { [$1] }
    | combinational_entry combinational_entry_list1 { $1::$2 }

concatenation
    : T_LBRACE expression_list1 T_RBRACE { $2 }

conditional_statement
    : T_if T_LPAREN expression T_RPAREN statement_or_null { ($3,$5,None) }
    | T_if T_LPAREN expression T_RPAREN statement_or_null
      T_else statement_or_null
	{ ($3,$5,Some($7)) }

constant_expression
    : expression { $1 }

continuous_assign
    : T_assign drive_strength_opt delay_opt list_of_assignments T_SEMICOLON
	{ ($2,$3,$4) }

delay
    : T_HASH number { NumDelay($2) }
    | T_HASH identifier { IDDelay($2) }
    | T_HASH T_LPAREN mintypmax_expression T_RPAREN
	{ MintypmaxDelay($3,None,None) }
    | T_HASH T_LPAREN mintypmax_expression T_COMMA
      mintypmax_expression T_RPAREN
	{ MintypmaxDelay($3,Some($5),None) }
    | T_HASH T_LPAREN mintypmax_expression T_COMMA
      mintypmax_expression T_COMMA mintypmax_expression T_RPAREN
	{ MintypmaxDelay($3,Some($5),Some($7)) }

delay_opt
    : { None }
    | delay { Some($1) }

delay_control
    : T_HASH number { NumDelayCtrl($2) }
    | T_HASH identifier { IDDelayCtrl($2) }
    | T_HASH T_LPAREN mintypmax_expression T_RPAREN { MintypmaxDelayCtrl($3) }

delay_or_event_control
    : delay_control { DelayCtrl($1) }
    | event_control { EventCtrl($1) }
    | T_repeat T_LPAREN expression T_RPAREN event_control
	{ RepeatEventCtrl($3,$5) }

delay_or_event_control_opt
    : { None }
    | delay_or_event_control { Some($1) }

description
    : module_declaration { ModuleDecl($1) }
    | udp_declaration { UdpDecl($1) }

description_list
    : { [] }
    | description description_list { $1::$2 }

disable_statement
    : T_disable name_of_block_or_task T_SEMICOLON { $2 }

drive_strength
    : T_LPAREN strength0 T_COMMA strength1 T_RPAREN { ($2,$4,true) }
    | T_LPAREN strength1 T_COMMA strength0 T_RPAREN { ($4,$2,false) }

drive_strength_opt
    : { None }
    | drive_strength { Some($1) }

edge
    : T_LPAREN level_symbol level_symbol T_RPAREN { Edge1($2,$3) }
    | edge_symbol { Edge2($1) }

edge_input_list
    : level_symbol_list edge level_symbol_list { ($1,$2,$3) }

edge_symbol
    : T_SymbolR { SymbolR }
    | T_SymbolF { SymbolF }
    | T_SymbolP { SymbolP }
    | T_SymbolN { SymbolN }
    | T_ASTERISK { SymbolA }

event_control
    : T_AT identifier { IDEventCtrl($2) }
    | T_AT T_LPAREN event_expression_or T_RPAREN { ExprEventCtrl($3) }

event_declaration
    : T_event name_of_event_list1 T_SEMICOLON { $2 }

event_expression
    : expression { ExprEventExpr($1) }
    | T_posedge scalar_event_expression { PosedgeEventExpr($2) }
    | T_negedge scalar_event_expression { NegedgeEventExpr($2) }

event_expression_or
    : event_expression { $1 }
    | event_expression_or T_or event_expression { OrEventExpr($1,$3) }

event_trigger
    : T_MGREATER name_of_event T_SEMICOLON { $2 }

expandrange
    : range { (None,$1) }
    | T_scalared range { (Some(true),$2) }
    | T_vectored range { (Some(false),$2) }

expandrange_opt
    : { None }
    | expandrange { Some($1) }

expression
    : primary { PrimaryExpr($1) }
    | unary_operator primary %prec UNARY_PREC { UnaryExpr($1,$2) }
    | expression T_PLUS expression { BinaryExpr(BAdd,$1,$3) }
    | expression T_MINUS expression { BinaryExpr(BSub,$1,$3) }
    | expression T_ASTERISK expression { BinaryExpr(BMul,$1,$3) }
    | expression T_SLASH expression { BinaryExpr(BDiv,$1,$3) }
    | expression T_PERCENT expression { BinaryExpr(BMod,$1,$3) }
    | expression T_EEQUAL expression { BinaryExpr(BEq,$1,$3) }
    | expression T_BEQUAL expression { BinaryExpr(BNEq,$1,$3) }
    | expression T_EEEQUAL expression { BinaryExpr(BCaseEq,$1,$3) }
    | expression T_BEEQUAL expression { BinaryExpr(BCaseNEq,$1,$3) }
    | expression T_AAMP expression { BinaryExpr(BLAnd,$1,$3) }
    | expression T_VVLINE expression { BinaryExpr(BLOr,$1,$3) }
    | expression T_LESS expression { BinaryExpr(BLess,$1,$3) }
    | expression T_LEQUAL expression { BinaryExpr(BLessEq,$1,$3) }
    | expression T_GREATER expression { BinaryExpr(BGreater,$1,$3) }
    | expression T_GEQUAL expression { BinaryExpr(BGreaterEq,$1,$3) }
    | expression T_AMP expression { BinaryExpr(BAnd,$1,$3) }
    | expression T_VLINE expression { BinaryExpr(BOr,$1,$3) }
    | expression T_CACCENT expression { BinaryExpr(BXor,$1,$3) }
    | expression T_TCACCENT expression { BinaryExpr(BXnor,$1,$3) }
    | expression T_CTILDE expression { BinaryExpr(BXnor,$1,$3) }
    | expression T_GGREATER expression { BinaryExpr(BRShift,$1,$3) }
    | expression T_LLESS expression { BinaryExpr(BLShift,$1,$3) }
    | expression T_QMARK expression T_COLON expression
	{ TernaryExpr($1,$3,$5) }
    | string { StringExpr($1) }

expression_list1
    : expression { [$1] }
    | expression T_COMMA expression_list1 { $1::$3 }

expression_opt
    : { None }
    | expression { Some($1) }

function
    : T_function range_or_type_opt name_of_function T_SEMICOLON
      function_item_declaration_list1 statement T_endfunction
	{ ($2,$3,$5,$6) }

function_call
    : name_of_function T_LPAREN expression_list1 T_RPAREN { FunCall($1,$3) }
    | name_of_system_function { SystemFunCall($1,[]) }
    | name_of_system_function T_LPAREN expression_list1 T_RPAREN
	{ SystemFunCall($1,$3) }

function_item_declaration
    : block_item_declaration { $1 }
    | input_declaration { InputDecl($1) }

function_item_declaration_list1
    : function_item_declaration { [$1] }
    | function_item_declaration function_item_declaration_list1 { $1::$2 }

gate_instance
    : T_LPAREN terminal_list1 T_RPAREN { (None,$2) }
    | name_of_gate_instance T_LPAREN terminal_list1 T_RPAREN { (Some($1),$3) }

gate_instance_list1
    : gate_instance { [$1] }
    | gate_instance T_COMMA gate_instance_list1 { $1::$3 }

gate_instantiation
    : gate_type gate_instance_list1 T_SEMICOLON
	{ ($1,None,None,$2) }
    | gate_type delay gate_instance_list1 T_SEMICOLON
	{ ($1,None,Some($2),$3) }
    | gate_type drive_strength delay_opt gate_instance_list1 T_SEMICOLON
	{ ($1,Some($2),$3,$4) }

gate_type
    : T_and { AndGate }
    | T_nand { NandGate }
    | T_or { OrGate }
    | T_nor { NorGate }
    | T_xor { XorGate }
    | T_xnor { XnorGate }
    | T_buf { BufGate }
    | T_not { NotGate }
    | T_bufif0 { Bufif0Gate }
    | T_bufif1 { Bufif1Gate }
    | T_notif0 { Notif0Gate }
    | T_notif1 { Notif1Gate }
    | T_nmos { NmosGate }
    | T_pmos { PmosGate }
    | T_rnmos { RnmosGate }
    | T_rpmos { RpmosGate }
    | T_tran { TranGate }
    | T_rtran { RtranGate }
    | T_tranif0 { Tranif0Gate }
    | T_tranif1 { Tranif1Gate }
    | T_rtranif0 { Rtranif0Gate }
    | T_rtranif1 { Rtranif1Gate }
    | T_cmos { CmosGate }
    | T_rcmos { RcmosGate }
    | T_pullup { PullupGate }
    | T_pulldown { PulldownGate }

identifier
    : single_identifier_list1 { $1 }

init_val
    : number
	{
	    let
		val t = String.lowercase($1)
	    in
		if t="1'b0" || t="1'b1" || t="1'bx" || t="1" || t="0" then $1
		else raise Parsing.Parse_error
	    end
	}

initial_statement
    : T_initial statement { $2 }

inout_declaration
    : T_inout range_opt list_of_variables T_SEMICOLON { ($2,$3) }

input_declaration
    : T_input range_opt list_of_variables T_SEMICOLON { ($2,$3) }

input_list
    : level_input_list { LevelInput($1) }
    | edge_input_list { EdgeInput($1) }

integer_declaration
    : T_integer list_of_register_variables T_SEMICOLON { $2 }

level_input_list
    : level_symbol_list1 { $1 }

level_symbol
    : T_Symbol0 { Symbol0 }
    | T_Symbol1 { Symbol1 }
    | T_SymbolX { SymbolX }
    | T_SymbolB { SymbolB }
    | T_QMARK { SymbolQ }

level_symbol_list
    : { [] }
    | level_symbol level_symbol_list { $1::$2 }

level_symbol_list1
    : level_symbol { [$1] }
    | level_symbol level_symbol_list1 { $1::$2 }

list_of_assignments
    : assignment_list1 { $1 }

list_of_module_connections
    : named_port_connection_list1 { NamedPortList($1) }
    | ordered_port_connection_list1 { OrderedPortList($1) }

list_of_param_assignments
    : param_assignment_list1 { $1 }

list_of_ports
    : T_LPAREN port_list1 T_RPAREN { $2 }

list_of_ports_opt
    : { None }
    | list_of_ports { Some($1) }

list_of_register_variables
    : register_variable_list1 { $1 }

list_of_variables
    : name_of_variable_list1 { $1 }

loop_statement
    : T_forever statement { Forever($2) }
    | T_repeat T_LPAREN expression T_RPAREN statement { Repeat($3,$5) }
    | T_while T_LPAREN expression T_RPAREN statement { While($3,$5) }
    | T_for T_LPAREN assignment T_SEMICOLON expression
      T_SEMICOLON assignment T_RPAREN statement
	{ For($3,$5,$7,$9) }

lvalue
    : identifier { IDLvalue($1) }
    | identifier T_LBRACKET expression T_RBRACKET { IDIndexLvalue($1,$3) }
    | identifier T_LBRACKET constant_expression T_COLON
      constant_expression T_RBRACKET
	{ IDRangeLvalue($1,$3,$5) }
    | concatenation { ConcatLvalue($1) }

mintypmax_expression
    : expression { ($1,None) }
    | expression T_COLON expression T_COLON expression { ($1,Some($3,$5)) }

module_declaration
    : module_keyword name_of_module list_of_ports_opt T_SEMICOLON
      module_item_list T_endmodule
	{ ($1,$2,$3,$5) }

module_instance
    : name_of_instance T_LPAREN list_of_module_connections T_RPAREN
	{ ($1,$3) }

module_instance_list1
    : module_instance { [$1] }
    | module_instance T_COMMA module_instance_list1 { $1::$3 }

module_instantiation
    : name_of_module parameter_value_assignment_opt
      module_instance_list1 T_SEMICOLON
	{ ($1,$2,$3) }

module_item
    : parameter_declaration { Declaration(ParamDecl($1)) }
    | input_declaration { Declaration(InputDecl($1)) }
    | output_declaration { Declaration(OutputDecl($1)) }
    | inout_declaration { Declaration(InoutDecl($1)) }
    | net_declaration { Declaration(NetDecl($1)) }
    | reg_declaration { Declaration(RegDecl($1)) }
    | integer_declaration { Declaration(IntegerDecl($1)) }
    | real_declaration { Declaration(RealDecl($1)) }
    | time_declaration { Declaration(TimeDecl($1)) }
    | realtime_declaration { Declaration(RealtimeDecl($1)) }
    | event_declaration { Declaration(EventDecl($1)) }
    | gate_instantiation { GateInst($1) }
    | udp_instantiation { UdpInst($1) }
    | module_instantiation { ModuleInst($1) }
    | parameter_override { ParamOverride($1) }
    | continuous_assign { ContAssign($1) }
    | specify_block { SpecifyBlock($1) }
    | initial_statement { InitialStmt($1) }
    | always_statement { AlwaysStmt($1) }
    | task { Task($1) }
    | function { Function($1) }

module_item_list
    : { [] }
    | module_item module_item_list { $1::$2 }

module_keyword
    : T_module { Module }
    | T_macromodule { MacroModule }

multiple_concatenation
    : T_LBRACE expression T_LBRACE expression_list1 T_RBRACE T_RBRACE
	{ $2::$4 }

name_of_block
    : single_identifier { $1 }

name_of_block_or_task
    : single_identifier { $1 }

name_of_event
    : single_identifier { $1 }

name_of_event_list1
    : name_of_event { [$1] }
    | name_of_event T_COMMA name_of_event_list1 { $1::$3 }

name_of_function
    : single_identifier { $1 }

name_of_gate_instance
    : single_identifier range_opt { ($1,$2) }

name_of_instance
    : single_identifier range_opt { ($1,$2) }

name_of_memory
    : single_identifier { $1 }

name_of_module
    : single_identifier { $1 }

name_of_port
    : single_identifier { $1 }

name_of_register
    : single_identifier { $1 }

name_of_system_function
    : system_identifier { $1 }

name_of_system_task
    : system_identifier { $1 }

name_of_task
    : single_identifier { $1 }

name_of_udp
    : single_identifier { $1 }

name_of_variable
    : single_identifier { $1 }

name_of_variable_list1
    : name_of_variable { [$1] }
    | name_of_variable T_COMMA name_of_variable_list1 { $1::$3 }

named_port_connection
    : T_DOT single_identifier T_LPAREN expression_opt T_RPAREN { ($2,$4) }

named_port_connection_list1
    : named_port_connection { [$1] }
    | named_port_connection T_COMMA named_port_connection_list1 { $1::$3 }

net_declaration
    : net_type expandrange_opt delay_opt list_of_variables T_SEMICOLON
	{ NetDecl1($1,$2,$3,$4) }
    | T_trireg charge_strength_opt expandrange_opt delay_opt
      list_of_variables T_SEMICOLON
	{ NetDecl2($2,$3,$4,$5) }
    | net_type expandrange_opt delay_opt
      list_of_assignments T_SEMICOLON
	{ NetDecl3($1,None,$2,$3,$4) }
    | net_type drive_strength expandrange_opt delay_opt
      list_of_assignments T_SEMICOLON
	{ NetDecl3($1,Some($2),$3,$4,$5) }

net_type
    : T_wire { Wire }
    | T_tri { Tri }
    | T_tri1 { Tri1 }
    | T_supply0 { NSupply0 }
    | T_wand { Wand }
    | T_triand { Triand }
    | T_tri0 { Tri0 }
    | T_supply1 { NSupply1 }
    | T_wor { Wor }
    | T_trior { Trior }

next_state
    : T_Symbol0 { Symbol0 }
    | T_Symbol1 { Symbol1 }
    | T_SymbolX { SymbolX }
    | T_MINUS { SymbolM }

non_blocking_assignment
    : lvalue T_LEQUAL delay_or_event_control_opt expression { ($1,$3,$4) }

number
    : T_NUM { $1 }

ordered_port_connection
    : expression_opt { $1 }

ordered_port_connection_list1
    : ordered_port_connection { [$1] }
    | ordered_port_connection T_COMMA ordered_port_connection_list1 { $1::$3 }

output_declaration
    : T_output range_opt list_of_variables T_SEMICOLON { ($2,$3) }

output_symbol
    : T_Symbol0 { Symbol0 }
    | T_Symbol1 { Symbol1 }
    | T_SymbolX { SymbolX }

output_terminal_name
    : name_of_variable { $1 }

par_block
    : T_fork statement_list T_join { (None,$2) }
    | T_fork T_COLON name_of_block block_item_declaration_list
      statement_list T_join
	{ (Some($3,$4),$5) }

param_assignment
    : identifier T_EQUAL constant_expression { ($1,$3) }

param_assignment_list1
    : param_assignment { [$1] }
    | param_assignment T_COMMA param_assignment_list1 { $1::$3 }

parameter_declaration
    : T_parameter list_of_param_assignments T_SEMICOLON { $2 }

parameter_override
    : T_defparam list_of_param_assignments T_SEMICOLON { $2 }

parameter_value_assignment
    : T_HASH T_LPAREN expression_list1 T_RPAREN { $3 }

parameter_value_assignment_opt
    : { None }
    | parameter_value_assignment { Some($1) }

port
    : port_expression_opt { (None,$1) }
    | T_DOT name_of_port T_LPAREN port_expression_opt T_RPAREN
	{ (Some($2),$4) }

port_list1
    : port { [$1] }
    | port T_COMMA port_list1 { $1::$3 }

port_expression
    : port_reference { [$1] }
    | T_LBRACE port_reference_list1 T_RBRACE { $2 }

port_expression_opt
    : { None }
    | port_expression { Some($1) }

port_reference
    : name_of_variable { ($1,None,None) }
    | name_of_variable T_LBRACKET constant_expression T_RBRACKET
	{ ($1,Some($3),None) }
    | name_of_variable T_LBRACKET constant_expression
      T_COLON constant_expression T_RBRACKET
	{ ($1,Some($3),Some($5)) }

port_reference_list1
    : port_reference { [$1] }
    | port_reference T_COMMA port_reference_list1 { $1::$3 }

primary
    : number { NumPrimary($1) }
    | identifier { IDPrimary($1) }
    | identifier T_LBRACKET expression T_RBRACKET { IDIndexPrimary($1,$3) }
    | identifier T_LBRACKET constant_expression T_COLON
      constant_expression T_RBRACKET
	{ IDRangePrimary($1,$3,$5) }
    | concatenation { ConcatPrimary($1) }
    | multiple_concatenation { MulConcatPrimary($1) }
    | function_call { FunCallPrimary($1) }
    | T_LPAREN mintypmax_expression T_RPAREN { MintypmaxPrimary($2) }

procedural_continuous_assignment
    : T_assign assignment { Assign($2) }
    | T_deassign lvalue { Deassign($2) }
    | T_force assignment { Force($2) }
    | T_release lvalue { Release($2) }

procedural_timing_control_statement
    : delay_control statement_or_null { (DelayCtrl($1),$2) }
    | event_control statement_or_null { (EventCtrl($1),$2) }

range
    : T_LBRACKET constant_expression T_COLON constant_expression T_RBRACKET
	{ ($2,$4) }

range_opt
    : { None }
    | range { Some($1) }

range_or_type
    : range { Range($1) }
    | T_integer { Integer }
    | T_real { Real }
    | T_realtime { Realtime }
    | T_time { Time }

range_or_type_opt
    : { None }
    | range_or_type { Some($1) }

real_declaration
    : T_real list_of_variables T_SEMICOLON { $2 }

realtime_declaration
    : T_realtime list_of_variables T_SEMICOLON { $2 }

reg_declaration
    : T_reg range_opt list_of_register_variables T_SEMICOLON { ($2,$3) }

register_variable
    : name_of_register { Register($1,None) }
    | name_of_register T_EQUAL expression { Register($1,Some($3)) }
    | name_of_memory T_LBRACKET constant_expression T_COLON
      constant_expression T_RBRACKET
	{ Memory($1,$3,$5) }

register_variable_list1
    : register_variable { [$1] }
    | register_variable T_COMMA register_variable_list1 { $1::$3 }

scalar_event_expression
    : expression { $1 }

seq_block
    : T_begin statement_list T_end { (None,$2) }
    | T_begin T_COLON name_of_block block_item_declaration_list
      statement_list T_end
	{ (Some($3,$4),$5) }

sequential_entry
    : input_list T_COLON state T_COLON next_state T_SEMICOLON { ($1,$3,$5) }

sequential_entry_list1
    : sequential_entry { [$1] }
    | sequential_entry sequential_entry_list1 { $1::$2 }

single_identifier
    : T_ID { $1 }

single_identifier_list1
    : single_identifier { [$1] }
    | single_identifier T_DOT single_identifier_list1 { $1::$3 }

source_text
    : description_list { $1 }

specify_block
    : T_specify specify_item_list T_endspecify { $2 }

specify_item
    : T_and { () }

specify_item_list
    : { [] }
    | specify_item specify_item_list { $1::$2 }

state
    : level_symbol { $1 }

statement
    : blocking_assignment T_SEMICOLON { BlockAssign($1) }
    | non_blocking_assignment T_SEMICOLON { NonBlockAssign($1) }
    | procedural_continuous_assignment T_SEMICOLON { ProcContAssign($1) }
    | procedural_timing_control_statement { ProcTimingCtrlStmt($1) }
    | conditional_statement { CondStmt($1) }
    | case_statement { CaseStmt($1) }
    | loop_statement { LoopStmt($1) }
    | wait_statement { WaitStmt($1) }
    | disable_statement { DisableStmt($1) }
    | event_trigger { EventTrigger($1) }
    | seq_block { SeqBlock($1) }
    | par_block { ParBlock($1) }
    | task_enable { TaskEnable($1) }
    | system_task_enable { SystemTaskEnable($1) }

statement_list
    : { [] }
    | statement statement_list { $1::$2 }

statement_or_null
    : statement { Some($1) }
    | T_SEMICOLON { None }

strength0
    : T_supply0 { SSupply0 }
    | T_strong0 { Strong0 }
    | T_pull0 { Pull0 }
    | T_weak0 { Weak0 }
    | T_highz0 { Highz0 }

strength1
    : T_supply1 { SSupply1 }
    | T_strong1 { Strong1 }
    | T_pull1 { Pull1 }
    | T_weak1 { Weak1 }
    | T_highz1 { Highz1 }

string
    : T_STR { $1 }

system_identifier
    : T_SID { $1 }

system_task_enable
    : name_of_system_task T_SEMICOLON { ($1,[]) }
    | name_of_system_task T_LPAREN expression_list1 T_RPAREN T_SEMICOLON
	{ ($1,$3) }

table_definition
    : T_table table_entries T_endtable { $2 }

table_entries
    : combinational_entry_list1 { CombEntry($1) }
    | sequential_entry_list1 { SeqEntry($1) }

task
    : T_task name_of_task T_SEMICOLON task_item_declaration_list
      statement_or_null T_endtask
	{ ($2,$4,$5) }

task_enable
    : name_of_task T_SEMICOLON { ($1,[]) }
    | name_of_task T_LPAREN expression_list1 T_RPAREN T_SEMICOLON { ($1,$3) }

task_item_declaration
    : block_item_declaration { $1 }
    | input_declaration { InputDecl($1) }
    | output_declaration { OutputDecl($1) }
    | inout_declaration { InoutDecl($1) }

task_item_declaration_list
    : { [] }
    | task_item_declaration task_item_declaration_list { $1::$2 }

terminal
    : expression { $1 }

terminal_list1
    : terminal { [$1] }
    | terminal T_COMMA terminal_list1 { $1::$3 }

time_declaration
    : T_time list_of_register_variables T_SEMICOLON { $2 }

udp_declaration
    : T_primitive name_of_udp T_LPAREN name_of_variable_list1 T_RPAREN
      T_SEMICOLON udp_port_declaration_list1 udp_initial_statement_opt
      table_definition T_endprimitive
	{ ($2,$4,$7,$8,$9) }

udp_initial_statement
    : T_initial output_terminal_name T_EQUAL init_val { ($2,$4) }

udp_initial_statement_opt
    : { None }
    | udp_initial_statement { Some($1) }

udp_instantiation
    : T_endcase { () }

udp_port_declaration
    : input_declaration { InputDecl($1) }
    | output_declaration { OutputDecl($1) }
    | reg_declaration { RegDecl($1) }

udp_port_declaration_list1
    : udp_port_declaration { $1 }
    | udp_port_declaration udp_port_declaration_list1 { $1::$2 }

unary_operator
    : T_PLUS { UPlus }
    | T_MINUS { UMinus }
    | T_BANG { ULNot }
    | T_TILDE { UNot }
    | T_AMP { UAnd }
    | T_TAMP { UNand }
    | T_VLINE { UOr }
    | T_TVLINE { UNor }
    | T_CACCENT { UXor }
    | T_TCACCENT { UXnor }
    | T_CTILDE { UXnor }

wait_statement
    : T_wait T_LPAREN expression T_RPAREN statement_or_null { ($3,$5) }

%%

(* vim: set ts=8 sw=4 sts=4 smarttab: *)
